package me.km.playerbank;

import java.util.HashMap;
import java.util.UUID;
import me.hammerle.snuviscript.code.ISnuviLogger;
import me.km.scoreboard.ScoreboardUtils;
import me.km.snuviscript.ScriptEvents;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.server.MinecraftServer;
import net.minecraftforge.eventbus.api.EventPriority;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.PlayerEvent;

public class PlayerManager
{
    private final HashMap<UUID, Integer> playerIds = new HashMap<>();
    
    private final HashMap<UUID, HashMap<String, Object>> playerData = new HashMap<>();
    private final HashMap<UUID, HashMap<String, Integer>> playerTickData = new HashMap<>();
    
    private final ISnuviLogger logger;
    private final IPlayerBank bank;
    private final MinecraftServer server;
    private final ScriptEvents events;
    
    public PlayerManager(ISnuviLogger logger, IPlayerBank bank, MinecraftServer server, ScriptEvents events) 
    {
        this.logger = logger;
        this.bank = bank;
        this.server = server;
        this.events = events;
    }
    
    public synchronized int getPlayerId(UUID uuid)
    {
        Integer i = playerIds.get(uuid);
        if(i != null)
        {
            return i;
        }
        return bank.getId(uuid);
    }
    
    @SubscribeEvent(priority = EventPriority.HIGH)
    public void onPlayerJoin(PlayerEvent.PlayerLoggedInEvent e)
    {
        PlayerEntity p = e.getPlayer();
        if(p == null)
        {
            return;
        }
        
        int id = bank.getId(p.getUniqueID());
        if(id == -1)
        {
            bank.add(p.getUniqueID(), p.getName().getUnformattedComponentText());
            logger.print(String.format("'%s' was added", p.getName()));
            
            id = bank.getId(p.getUniqueID());
            if(id == -1)
            {
                // this should never happen
                logger.print(String.format("'%s' has no id", p.getName()));
            }
            else
            {
                playerIds.put(p.getUniqueID(), id);
            }
        }
        else
        {
            playerIds.put(p.getUniqueID(), id);
            bank.changeName(p.getUniqueID(), p.getName().getUnformattedComponentText());
        }
    }
    
    @SubscribeEvent(priority = EventPriority.LOW)
    public void onPlayerLeaveCleanup(PlayerEvent.PlayerLoggedOutEvent e)
    {      
        UUID uuid = e.getPlayer().getUniqueID();
        ScoreboardUtils.remove(uuid);
        playerIds.remove(uuid);
        playerData.remove(uuid);
        playerTickData.remove(uuid);
    }
    
    public void tickPlayerData()
    {
        playerTickData.forEach((uuid, map) -> 
        {
            PlayerEntity p = server.getPlayerList().getPlayerByUUID(uuid);
            if(p == null)
            {
                return;
            }
            map.entrySet().removeIf(entry -> 
            {
                int time = entry.getValue() - 1;
                if(time <= 0)
                {
                    if(time == 0)
                    {
                        events.onPlayerDataTick(p, entry.getKey());
                    }
                    return true;
                }
                entry.setValue(time);
                return false;
            });
        });
    }
    
    public void setPlayerVar(UUID uuid, String varname, Object value)
    {
        HashMap<String, Object> map = playerData.get(uuid);
        if(map == null)
        {
            map = new HashMap<>();
            playerData.put(uuid, map);
        }
        map.put(varname, value);
    }
    
    public Object getPlayerVar(UUID uuid, String varname)
    {
        HashMap<String, Object> map = playerData.get(uuid);
        if(map == null)
        {
            return null;
        }
        return map.get(varname);
    }
    
    public void setPlayerTimer(UUID uuid, String varname, int time)
    {
        HashMap<String, Integer> map = playerTickData.get(uuid);
        if(map == null)
        {
            map = new HashMap<>();
            playerTickData.put(uuid, map);
        }
        map.put(varname, time);
    }
    
    public int getPlayerTimer(UUID uuid, String varname)
    {
        HashMap<String, Integer> map = playerTickData.get(uuid);
        if(map == null)
        {
            return -1;
        }
        return map.getOrDefault(varname, -1);
    }
    
    public void clearPlayerData(UUID uuid)
    {
        playerTickData.remove(uuid);
        playerData.remove(uuid);
    }
}
